using BPA.SAAS.Manage.Application.Auth.Dtos;
using BPA.SAAS.Manage.Core.Base;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.SAAS.Manage.Application.Auth
{
    public interface IAuthService
    {
        Task<LoginOutInfo> Login(LoginInput input);
        Task<LoginOutput> GetLoginUserAsync();
        Task LogoutAsync();

        #region 添加平台授权

        /// <summary>
        /// 分页
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        Task<PageUtil> PageAuthorization(PageInputBase input);

        /// <summary>
        /// 添加授权码
        /// </summary>
        /// <returns></returns>
        Task<bool> AddAuthorization(CreateOrUpDatePlatformAuthorizationDto input);

        /// <summary>
        /// 修改授权码
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        Task<bool> UpdateAuthorization(string id);


        /// <summary>
        /// 修改授权码授权时间
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        Task<bool> UpdateAuthTime(CreateOrUpDatePlatformAuthorizationDto input);


        /// <summary>
        /// 删除授权
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        Task<bool> DelAuthorization(string id);

        #endregion
    }
}
